﻿// m: Цепь
// Версия 2.0.1 от 05.11.2014

function MainChain(canvas) {

    // Предварительные установки

    var context = canvas.getContext("2d");                  // на context происходит рисование
    document.oncontextmenu=function(e){return false};       // блокировка контекстного меню

    var Pi = 3.1415926;                   // число "пи"

    var m0 = 1;                           // масштаб массы
    var T0 = 1;                           // масштаб времени (период колебаний исходной системы)
    var a0 = 1;                           // масштаб расстояния (диаметр шара)

    var k0 = 2 * Pi / T0;                 // масштаб частоты
    var C0 = m0 * k0 * k0;                // масштаб жесткости

    // *** Задание физических параметров ***

    var Ny = 5;						    // Число шаров, помещающихся по вертикали в окно (задает размер шара относительно размера окна)
    var scale = canvas.height / Ny / a0;  // масштабный коэффициент для перехода от расчетных к экранным координатам
    var m = 1 * m0;                 	    // масса
    var C = 1 * C0;                 	    // жесткость
    var num = 24;                           // начальное количество частиц

    // *** Задание вычислительных параметров ***

    var fps = 50;             	        // frames per second - число кадров в секунду (качечтво отображения)
    var spf = 100;              		    // steps per frame   - число шагов интегрирования между кадрами (скорость расчета)
    var dt  = 0.045 * T0 / fps;      	    // шаг интегрирования (качество расчета)

    // Выполнение программы

    var w = canvas.width / scale;		    // ширина окна в расчетных координатах
    var h = canvas.height / scale;		    // высота окна в расчетных координатах

    // Генерорование начальных условий

    var balls;                              // массив шаров
    var uGraph; var vGraph;                 // переменные графиков
    var r, rScale, pDist, hC, steps;        // описание дано далее
    var rScale13, rScaleShift;              // ___в целях оптимизации___
    MainChain.prototype.setNum = function(n){num = n};       // задать новое количество частиц
    MainChain.prototype.newSystem = function(conf){
        MainChain.prototype.actualConf = conf;
        r = a0*10/num;               	    // радиус частицы в расчетных координатах
        rScale = r * scale;                 // радиус частицы в экранных координатах
        pDist = w/num;                      // расстояние между шарами (в начальном положении)
        hC = h/2 * scale;                   // высота цепи
        rScale13 = rScale*1.3;              // ___в целях оптимизации___
        rScaleShift = rScale/5.0;           // ___в целях оптимизации___
        steps = 0;                          // количество шагов интегрирования

        balls = [];
        for (var i = 1; i < num + 1; i++) {
            var b = [];

            b.x0 = pDist*(0.5 + i-1);       // расчетные координаты начального положения шара
            b.fx = 0;                       // сила, действующая на шар
            b.vx = 0;                       // скорость
            b.uu = 0;                       // расчетное смещение шара относительно начального положения

            conf(b, i);                     // конфигурации начальных условий заданы ниже

            b.xx = b.x0 + b.uu;             // расчетные координаты положения шара
            b.x = b.xx*scale;               // экранные координаты шара

            balls[i] = b;                   // добавить элемент в конец массива
        }

        // здесь задается периодическая система
        balls[0] = balls[num];
        balls[num+1] = balls[1];

        // уравновешивание суммарной скорости по оси х (чтобы частицы не улетали в сторону)
        var sumVx = 0;
        for (var i0 = 1; i0 < num+1; i0++) sumVx += balls[i0].vx;
        var vxAverage = sumVx/num;
        for (var i1 = 1; i1 < num+1; i1++) balls[i1].vx -= vxAverage;

        // высота оси у на графике u(n)
        var sumVx2 = 0;
        for (var i2 = 1; i2 < num+1; i2++) sumVx2 += balls[i2].vx*balls[i2].vx;
        var sigma2 = sumVx2 / num;
        var omega2 = C/m;
        var yLabel = 2*Math.sqrt(sigma2/omega2) *(num/8);     // в конце - коэффициент

        uGraph = new TM_graph(              // определить график
            "#uGraph",                      // на html-элементе #uGraph
            "n", "u",                       // подписи на осях
            null                            // в данном типе графика не используется
            ,-yLabel, +yLabel, yLabel/4     // мин. значение оси Y, макс. значение оси Y, шаг по оси Y
        );

        vGraph = new TM_graph(              // определить график
            "#vGraph",                      // на html-элементе #vGraph
            "n", "v",                       // подписи на осях
            null                            // в данном типе графика не используется
            ,-1.2, 1.2, 1.2/4               // мин. значение оси Y, макс. значение оси Y, шаг по оси Y
        );
    };

    // настройки для конфигураций
    var v0 = 1*a0/T0;                       // conf_random - начальный разброс скоростей
    var sinNum = 2;                         // conf_sin - количество периодов синуса в цепи
    var hillDiv = 1/4;                      // conf_hill - часть (доля) цепи, которую занимает "холм"

    // конфигурации
    MainChain.prototype.conf_random = function(b){b.vx = v0*(2*Math.random()-1);};
    MainChain.prototype.conf_sin = function(b, i){b.vx = Math.sin(2*Pi * i/num*sinNum);};
    MainChain.prototype.conf_one = function(b, i){if (i == Math.ceil(num/2)) b.vx = 0.5; else b.vx = 0;};
    MainChain.prototype.conf_stair2 = function(b, i){if (i%4 == 0 || (i-1)%4 == 0) b.vx = 1; else b.vx = 0;};
    MainChain.prototype.conf_stair3 = function(b, i){if (i%6 == 0 || (i-1)%6 == 0 || (i-2)%6 == 0) b.vx = 1; else b.vx = 0;};
    MainChain.prototype.conf_hill = function(b, i){
        var nd2 = num*hillDiv/2;
        if (i <= nd2*2) b.vx = (nd2*nd2 - (i-nd2)*(i-nd2))/(nd2*nd2);
        else b.vx = 0;
    };

    // Основной цикл программы

    function control() {
        physics();
        draw();
    }

    // Расчетная часть программы

    function physics(){
        for (var s=1; s<=spf; s++) {// то, что происходит каждый шаг времени
            for (var i=1; i<balls.length-1; i++){
                balls[i].fx = C*(balls[i+1].uu - 2*balls[i].uu + balls[i-1].uu);
                balls[i].vx += balls[i].fx / m * dt;
                balls[i].xx += balls[i].vx * dt;

                balls[i].x = balls[i].xx * scale;
            }
            // присваиваем новые перемещения
            for (var i2=1; i2 < balls.length-1; i2++) balls[i2].uu = balls[i2].xx - balls[i2].x0;
        }
        steps++;

        // для графика создаем массив пар значений [x, y]
        var uData = []; var vData = [];
        for (var i0=1; i0<balls.length-1; i0++) uData[i0] = [i0, balls[i0].uu];
        for (var i1=1; i1<balls.length-1; i1++) vData[i1] = [i1, balls[i1].vx];

        if (steps % 4 == 0) uGraph.graph(uData);   // подаем данные на график
        if (steps % 4 == 0) vGraph.graph(vData);   // подаем данные на график
    }

    // Рисование

    function draw(){
        context.clearRect(0, 0, w*scale, h*scale);          // очистить экран
        for (var i = 0; i < balls.length; i++){
            var b = balls[i];

            // расчет градиента нужно проводить для каждого шара
            var gradient=context.createRadialGradient(b.x, hC, rScale13, b.x-rScaleShift, hC+rScaleShift,0);
            gradient.addColorStop(0,"#0000bb");
            gradient.addColorStop(1,"#44ddff");
            context.fillStyle=gradient;

            context.beginPath();
            context.arc(b.x, hC, r*scale,0,2*Math.PI);
            context.closePath();
            context.fill();
        }
    }

    // Запуск системы

    MainChain.prototype.newSystem(MainChain.prototype.conf_hill);
    setInterval(control, 1000/fps);
}